# Function Specification

The scope of obuspa is to expose the libbbfdm APIs using uspd over ubus.
Following are the commands used by obuspa for various operations. The overall
functionality can be checked through cli option available on obuspa.

NOTE: Please, note that the output below is for cli commands. And obuspa
agent process same commands over STOMP or CoAP connection with controller.


```
root@iopsys:~# obuspa -h
USAGE: obuspa options
--help (-h)       Displays this help
--log (-l)        Sets the destination for debug logging. Default is 'stdout'. Can also use 'syslog' or a filename
--dbfile (-f)     Sets the path of the file to store the database in (default=/tmp/usp.db)
--verbose (-v)    Sets the debug verbosity log level: 0=Off, 1=Error(default), 2=Warning, 3=Info
--prototrace (-p) Enables trace logging of the USP protocol messages
--authcert (-a)   Sets the path of the PEM formatted file containing a client certificate and private key to authenticate this device with
--truststore (-t) Sets the path of the PEM formatted file containing trust store certificates
--resetfile (-r)  Sets the path of the text file containing factory reset parameters
--interface (-i)  Sets the name of the networking interface to use for USP communication
--meminfo (-m)    Collects and prints information useful to debugging memory leaks
--error (-e)      Enables printing of the callstack whenever an error is detected
--command (-c)    Sends a CLI command to the running USP Agent and prints the response
                  To get a list of all CLI commands use '-c help'
```

# Commands
* [help](#help)
* [version](#version)
* [get](#get)
* [set](#set)
* [add](#add)
* [del](#del)
* [operate](#operate)
* [instances](#instances)
* [show](#show)
* [dump](#dump)
* [perm](#perm)
* [dbget](#dbget)
* [dbset](#dset)
* [dbdel](#dbdel)
* [verbose](#verbose)
* [prototrace](#prototrace)

### help 

To invoke help options to an obuspa agent through cli -c can be used.

````bash
root@iopsys:~# obuspa -c help 
Running command on database /tmp/usp.db
USP_REGISTER_Object_UniqueKey: Parameter 'SourceAddress' in unique key is not a child of 'Device.DHCPv6.Server.Pool.{i}.Client.{i}'
USP_REGISTER_Object_UniqueKey: Parameter 'Queue' in unique key is not a child of 'Device.Ethernet.RMONStats.{i}'
USP_REGISTER_Object_UniqueKey: Invalid parameters
USP_REGISTER_Object_UniqueKey: Parameter 'Profile' in unique key is not a child of 'Device.WiFi.EndPoint.{i}.Profile.{i}'
Valid commands:
   help
   version
   get [path-expr]
   set [path-expr] [value]
   add [object]
   del [path-expr]
   operate [operation]
   instances [path-expr]
   show ['datamodel' | 'database' ]
   dump ['connections' | 'memory' | 'mdelta' | 'subscriptions' | 'instances' ]
   perm [parameter or object]
   dbget [parameter]
   dbset [parameter] [value]
   dbdel [parameter]
   verbose [level]
   prototrace [enable]
   stop
````

| Method      								|Function ID	|
| :--- 	  									| :---        	|
| [help](#help)				                | 1				|
| [version](#version)				        | 2				|
| [get](#get)					            | 3				|
| [set](#set)		 		                | 4				|
| [add](#add)			                    | 5   			|
| [del](#del)		                        | 6 			|
| [operate](#operate)		 		        | 7				|
| [instances](#instances)					| 8				|
| [show](#show)				                | 9				|
| [dump](#dump)				 		        | 10			|
| [dbget](#dbget)	                        | 11			|
| [dbset](#dbset)		                    | 12			|
| [dbdel](#dbdel)		                    | 13			|
| [verbose](#verbose)			            | 14			|
| [prototrace](#prototrace)		            | 15			|

#### Methods

Method descriptions of the obuspa.


##### help

Exposes methods to operate obuspa.


##### version 

dump version of currently installed obuspa.


##### get

Exposes all information regarding various registered schema parameters.


##### set

Set information regarding various registered schema parameters.


##### add

Add object parameter to a specific registered schema.


##### del

Del object parameter to a specific registered schema.


##### operate

Execute various synch/asynch operations e.g., IPPing(), NeighbourDiagnostics() etc.


##### instances

Get all the instances for specified schema path.


##### show

Show all datamodel or database depending upon parameter specified.


##### dump

Dump information regarding ['connections' | 'memory' | 'mdelta' | 'subscriptions' | 'instances' ] based on specified options.


##### dbget

To fetch information of any database parameter.


##### dbset

To set value of a database parameter.


##### dbidel

To set value of a database parameter.


##### verbose

To set log level for error tracing and debugging.


##### prototrace 

To set log level for error tracing and debugging.


